from inicie.cropimage.tests.base import IntegrationTestCase
from Products.CMFCore.utils import getToolByName


class TestCase(IntegrationTestCase):
    """TestCase for Plone setup."""

    def setUp(self):
        self.portal = self.layer['portal']
        self.installer = getToolByName(self.portal, 'portal_quickinstaller')

    def test_is_inicie_cropimage_installed(self):
        self.failUnless(self.installer.isProductInstalled('inicie.cropimage'))

    def test_ids_registry(self):
        """Test if inicie.cropimage.ids is properly registered."""
        from zope.component import getUtility
        from plone.registry.interfaces import IRegistry
        registry = getUtility(IRegistry)
        self.assertEqual(len(registry['inicie.cropimage.ids']), 0)
        field = registry.records['inicie.cropimage.ids'].field
        self.assertEqual(field.title, u'Identifications')
        self.assertEqual(field.description, u'ID name for CSS.')
        from plone.registry.field import TextLine
        isinstance(field.value_type, TextLine)

    ## controlpanel.xml
    def test_controlpanel(self):
        controlpanel = getToolByName(self.portal, 'portal_controlpanel')
        action = [
            action for action in controlpanel.listActions() if action.id == 'crop_image_registry'
        ][0]
        self.assertEqual(action.title, 'Crop Image Registry')
        self.assertEqual(action.appId, 'inicie.cropimage')
        self.assertEqual(action.action.text, 'string:${portal_url}/@@crop-image-controlpanel')
        self.assertEqual(action.icon_expr.text, 'string:$portal_url/cut_icon.png')
        self.assertTrue(action.visible)
        self.assertEqual(action.permissions, ('Manage portal',))

    ## actions.xml
    def test_crop_image(self):
        actions = getToolByName(self.portal, 'portal_actions')
        action = actions.object_buttons.crop_image
        self.assertEqual('Crop Image', action.getProperty('title'))
        self.assertEqual(
            'string:${globals_view/getCurrentObjectUrl}/@@crop-image',
            action.getProperty('url_expr')
        )
        self.assertEqual(
            'python: object.restrictedTraverse("has-image")()',
            action.getProperty('available_expr')
        )
        self.assertEqual(('Modify portal content',), action.getProperty('permissions'))
        self.assertEqual(True, action.getProperty('visible'))

    ## jsregistry.xml
    def test_jsregistry(self):
        javascripts = getToolByName(self.portal, 'portal_javascripts')
        res = '++resource++inicie.cropimage.javascripts/jquery.Jcrop.js'
        self.assertTrue(res in javascripts.getResourceIds())
        item = javascripts.getResource(res)
        self.assertEqual(item.getCompression(), 'none')

    ## cssregistry.xml
    def test_cssregistry(self):
        css = getToolByName(self.portal, 'portal_css')
        res = '++resource++inicie.cropimage.stylesheets/main.css'
        self.assertTrue(res in css.getResourceIds())

    def test_uninstall(self):
        self.installer.uninstallProducts(['inicie.cropimage'])
        self.failIf(self.installer.isProductInstalled('inicie.cropimage'))
        from zope.component import getUtility
        from plone.registry.interfaces import IRegistry
        registry = getUtility(IRegistry)
        ## inicie.cropimage.banks should be uninstalled form registry.
        self.assertRaises(KeyError, lambda: registry['inicie.cropimage.ids'])
        controlpanel = getToolByName(self.portal, 'portal_controlpanel')
        ids = [action.id for action in controlpanel.listActions()]
        ## crop_image_registry configlet should be uninstalled from controlpanel
        self.assertTrue('crop_image_registry' not in ids)
        ## crop_image should be uninstalled from actions
        actions = getToolByName(self.portal, 'portal_actions')
        self.assertFalse(hasattr(actions.object_buttons, 'crop_image'))
        ## jsregisty.xml should be uninstalled.
        javascripts = getToolByName(self.portal, 'portal_javascripts')
        res = '++resource++inicie.cropimage.javascripts/jquery.Jcrop.js'
        self.assertFalse(res in javascripts.getResourceIds())
        ## cssregistry.xml should be uninstalled.
        css = getToolByName(self.portal, 'portal_css')
        res = '++resource++inicie.cropimage.stylesheets/main.css'
        self.assertFalse(res in css.getResourceIds())

    def test_browserlayer(self):
        from inicie.cropimage.browser.interfaces import IInicieCropimageLayer
        from plone.browserlayer import utils
        self.failUnless(IInicieCropimageLayer in utils.registered_layers())
